@@ -346,6 +346,36 @@ usr/bin/frob                                            foo/frob
 
         self.assertEqual(impl.get_file_package(file), pkg)
 
+    def test_mirror_from_apt_sources(self):
+        s = os.path.join(self.workdir, 'sources.list')
+
+        # valid file, should grab the first mirror
+        with open(s, 'w') as f:
+            f.write('''# some comment
+deb-src http://source.mirror/foo tuxy main
+deb http://binary.mirror/tuxy tuxy main
+deb http://secondary.mirror tuxy extra
+''')
+            f.flush()
+            self.assertEqual(impl._get_primary_mirror_from_apt_sources(s),
+                             'http://binary.mirror/tuxy')
+
+        # valid file with options
+        with open(s, 'w') as f:
+            f.write('''# some comment
+deb-src http://source.mirror/foo tuxy main
+deb [arch=flowerpc,leghf] http://binary.mirror/tuxy tuxy main
+deb http://secondary.mirror tuxy extra
+''')
+            f.flush()
+            self.assertEqual(impl._get_primary_mirror_from_apt_sources(s),
+                             'http://binary.mirror/tuxy')
+
+        # empty file
+        with open(s, 'w') as f:
+            f.flush()
+        self.assertRaises(SystemError, impl._get_primary_mirror_from_apt_sources, s)
+
     def test_get_modified_conffiles(self):
         '''get_modified_conffiles()'''
 
